/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is property of Nordic Semiconductor ASA.
 * Terms and conditions of usage are described in detail in NORDIC
 * SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

#include "nrf.h"
#include <stdbool.h>
#include <stdint.h>
//#include "bsp.h"              //include if you want to use board defines (LED_1, BUTTON_1, etc.)
#include "nrf_gpio.h"
#include "nrf_delay.h"
//#include "math.h"             //include if you want to use math.h functions.

// Peripheral channel assignments
#define GPIO_GPIOTE_CH_1    0
#define GPIO_GPIOTE_CH_2    1
#define GPIO_PPI_CH_A       0
#define GPIO_PPI_CH_B       1
#define GPIO_PPI_CH_C       2
#define GPIO_PPI_CH_D       3
#define GPIO_TIMER_CC_NUM_1 0
#define GPIO_TIMER_CC_NUM_2 1
#define GPIO_TIMER_CC_NUM_3 2
#define GPIO_TIMER_CC_NUM_4 3


// TIMER3 reload value. 
#define TIMER_RELOAD        16665 // = 3333*5us
// The timer CC register used to reset the timer. Be aware that not all timers in the nRF52 have 6 CC registers.
#define TIMER_RELOAD_CC_NUM 5


// This function initializes timer 3 with the following configuration:
// 24-bit, base frequency 16 MHz, auto clear on COMPARE5 match (CC5 = TIMER_RELOAD)
void timer_init()
{
    NRF_TIMER3->BITMODE                 = TIMER_BITMODE_BITMODE_24Bit << TIMER_BITMODE_BITMODE_Pos;
    NRF_TIMER3->PRESCALER               = 4;
    NRF_TIMER3->SHORTS                  = TIMER_SHORTS_COMPARE0_CLEAR_Msk << TIMER_RELOAD_CC_NUM;
    NRF_TIMER3->MODE                    = TIMER_MODE_MODE_Timer << TIMER_MODE_MODE_Pos;
    NRF_TIMER3->CC[TIMER_RELOAD_CC_NUM] = TIMER_RELOAD;    
    
    NRF_TIMER3->CC[GPIO_TIMER_CC_NUM_1] = 15;  // The gpio will be set to high after the timer reaches CC[GPIO_GPIOTE_CH].
    NRF_TIMER3->CC[GPIO_TIMER_CC_NUM_2] = 255;  // The gpio will be set to low  after the timer reaches CC[GPIO_GPIOTE_CH].
    NRF_TIMER3->CC[GPIO_TIMER_CC_NUM_3] = 300;
    NRF_TIMER3->CC[GPIO_TIMER_CC_NUM_4] = 510;
}


// Starts TIMER3
void timer_start()
{
    NRF_TIMER3->TASKS_START = 1;
}


// This function sets up TIMER3, the PPI and the GPIOTE modules to configure a single GPIOTE channel
// Timer CC num, PPI channel nums and GPIOTE channel num is defined at the top of this file
void gpio_init(uint32_t pinselect)
{  
    NRF_GPIOTE->CONFIG[GPIO_GPIOTE_CH_1] = GPIOTE_CONFIG_MODE_Task << GPIOTE_CONFIG_MODE_Pos | 
                                         GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos | 
                                         pinselect << GPIOTE_CONFIG_PSEL_Pos | 
                                         GPIOTE_CONFIG_OUTINIT_High << GPIOTE_CONFIG_OUTINIT_Pos;

    NRF_PPI->CH[GPIO_PPI_CH_A].EEP = (uint32_t)&NRF_TIMER3->EVENTS_COMPARE[GPIO_TIMER_CC_NUM_1];
    NRF_PPI->CH[GPIO_PPI_CH_A].TEP = (uint32_t)&NRF_GPIOTE->TASKS_SET[GPIO_GPIOTE_CH_1];
    NRF_PPI->CH[GPIO_PPI_CH_B].EEP = (uint32_t)&NRF_TIMER3->EVENTS_COMPARE[GPIO_TIMER_CC_NUM_2];
    NRF_PPI->CH[GPIO_PPI_CH_B].TEP = (uint32_t)&NRF_GPIOTE->TASKS_CLR[GPIO_GPIOTE_CH_1];
    
//    NRF_PPI->CHENSET               = (1 << GPIO_PPI_CH_A) | (1 << GPIO_PPI_CH_B);
    
    
    NRF_GPIOTE->CONFIG[GPIO_GPIOTE_CH_2] = GPIOTE_CONFIG_MODE_Task << GPIOTE_CONFIG_MODE_Pos | 
                                         GPIOTE_CONFIG_POLARITY_Toggle << GPIOTE_CONFIG_POLARITY_Pos | 
                                         pinselect+1 << GPIOTE_CONFIG_PSEL_Pos | 
                                         GPIOTE_CONFIG_OUTINIT_High << GPIOTE_CONFIG_OUTINIT_Pos;
    
    NRF_PPI->CH[GPIO_PPI_CH_C].EEP = (uint32_t)&NRF_TIMER3->EVENTS_COMPARE[GPIO_TIMER_CC_NUM_3];
    NRF_PPI->CH[GPIO_PPI_CH_C].TEP = (uint32_t)&NRF_GPIOTE->TASKS_SET[GPIO_GPIOTE_CH_2];
    NRF_PPI->CH[GPIO_PPI_CH_D].EEP = (uint32_t)&NRF_TIMER3->EVENTS_COMPARE[GPIO_TIMER_CC_NUM_4];
    NRF_PPI->CH[GPIO_PPI_CH_D].TEP = (uint32_t)&NRF_GPIOTE->TASKS_CLR[GPIO_GPIOTE_CH_2];
    
    NRF_PPI->CHENSET               = (1 << GPIO_PPI_CH_A) | (1 << GPIO_PPI_CH_B) | (1 << GPIO_PPI_CH_C) | (1 << GPIO_PPI_CH_D);    
    
}



//void gpio_set_ticks(uint32_t ticks)
//{
//    NRF_TIMER3->CC[GPIO_TIMER_CC_NUM_1] = ticks;
//}


int main(void)
{
    
    NRF_CLOCK->TASKS_HFCLKSTART = 1;
    while (!NRF_CLOCK->EVENTS_HFCLKSTARTED)
    {
        //wait for the xtal to start.
        //(The HFINT is used in the meantime, so you don't really need to wait)
    }
    
    
    // Initialize the timer
    timer_init();
    
    
    //gpio_init(LED_1); // pin 17 is LED 1. include bsp.h to use the LED_1 macro. Removed to easier port to other IDEs.
    gpio_init(17); 
    
    // Start the timer
    timer_start();

    while (true)
    {
        // do nothing
    }
}
